package ru.dbotthepony.kstarbound.defs

import java.io.DataInputStream
import java.io.DataOutputStream

// original game has MVariant here
// MVariant prepends InvalidValue to Variant<> template
// Variant<> itself works with LookupTypeIndex<MatchType, 0, FirstType, RestTypes...>
// 0 is responsible for holding current template comparison check
// typedef MVariant<WarpToWorld, WarpToPlayer, WarpAlias> WarpAction;
// -> Variant<InvalidType, WarpToWorld, WarpToPlayer, WarpAlias> WarpAction
// hence WarpToWorld has index 1, WarpToPlayer 2, WarpAlias 3

sealed class AbstractWarpTarget {
	abstract fun write(stream: DataOutputStream, isLegacy: Boolean)

	companion object {
		fun read(stream: DataInputStream, isLegacy: Boolean): AbstractWarpTarget {
			return when (stream.readUnsignedByte()) {
				3 -> {
					when (stream.readInt()) {
						0 -> WarpAlias.Return
						1 -> WarpAlias.OrbitedWorld
						2 -> WarpAlias.OwnShip
						else -> throw IllegalArgumentException()
					}
				}

				else -> throw IllegalArgumentException()
			}
		}
	}
}

sealed class WarpAlias(val index: Int) : AbstractWarpTarget() {
	override fun write(stream: DataOutputStream, isLegacy: Boolean) {
		stream.write(3)
		// because it is defined as enum class WarpAlias, without specifying uint8_t as type
		stream.writeInt(index)
	}

	object Return : WarpAlias(0)
	object OrbitedWorld : WarpAlias(1)
	object OwnShip : WarpAlias(2)
}
