package ru.dbotthepony.kstarbound.defs.animation

import com.google.common.collect.ImmutableMap
import com.google.gson.JsonObject
import it.unimi.dsi.fastutil.objects.Object2ObjectAVLTreeMap
import ru.dbotthepony.kstarbound.json.builder.IStringSerializable
import ru.dbotthepony.kstarbound.json.builder.JsonFactory

@JsonFactory
data class AnimatedPartsDefinition(
	val stateTypes: ImmutableMap<String, StateType> = ImmutableMap.of(),
	val parts: ImmutableMap<String, Part> = ImmutableMap.of(),
) {
	enum class AnimationMode(override val jsonName: String) : IStringSerializable {
		END("end"),
		TRANSITION("transition"),
		LOOP("loop");
	}

	@JsonFactory
	data class StateType(
		val default: String = "",
		val priority: Double = 0.0,
		val enabled: Boolean = true,
		val states: ImmutableMap<String, State> = ImmutableMap.of(),
		val properties: JsonObject = JsonObject(),
	) {
		@JsonFactory
		data class State(
			val frames: Int = 1,
			val cycle: Double = 1.0,
			val mode: AnimationMode = AnimationMode.END,
			val transition: String = "",
			val properties: JsonObject = JsonObject(),
			val frameProperties: JsonObject = JsonObject(),
		)
	}

	@JsonFactory
	data class Part(
		val properties: JsonObject = JsonObject(),
		val partStates: ImmutableMap<String, ImmutableMap<String, State>> = ImmutableMap.of(),
	) {
		@JsonFactory
		data class State(
			val properties: JsonObject = JsonObject(),
			val frameProperties: JsonObject = JsonObject(),
		)
	}
}