package ru.dbotthepony.kstarbound.world.api

import ru.dbotthepony.kstarbound.Registries
import ru.dbotthepony.kstarbound.Registry
import ru.dbotthepony.kstarbound.defs.tile.BuiltinMetaMaterials
import ru.dbotthepony.kstarbound.defs.tile.MaterialModifier
import ru.dbotthepony.kstarbound.defs.tile.TileDefinition
import java.io.DataInputStream

data class MutableTileState(
	override var material: Registry.Entry<TileDefinition> = BuiltinMetaMaterials.NULL,
	override var modifier: Registry.Entry<MaterialModifier>? = null,
	override var color: TileColor = TileColor.DEFAULT,
	override var hueShift: Float = 0f,
	override var modifierHueShift: Float = 0f,
) : AbstractTileState() {
	override fun immutable(): ImmutableTileState {
		return POOL.intern(ImmutableTileState(material, modifier, color, hueShift, modifierHueShift))
	}

	override fun mutable(): MutableTileState {
		return this
	}

	fun setHueShift(value: Int): MutableTileState {
		if (value < 0) {
			hueShift = 0f
		} else if (value >= 255) {
			hueShift = 360f
		} else {
			hueShift = value / 255f * 360f
		}

		return this
	}

	fun setModHueShift(value: Int): MutableTileState {
		if (value < 0) {
			modifierHueShift = 0f
		} else if (value >= 255) {
			modifierHueShift = 360f
		} else {
			modifierHueShift = value / 255f * 360f
		}

		return this
	}

	fun read(stream: DataInputStream): MutableTileState {
		material = Registries.tiles[stream.readUnsignedShort()] ?: BuiltinMetaMaterials.EMPTY
		setHueShift(stream.read())
		color = TileColor.of(stream.read())
		modifier = Registries.tileModifiers[stream.readUnsignedShort()]
		setModHueShift(stream.read())
		return this
	}
}
